@@ -299,8 +299,8 @@ def file_open(filename, type, parent, print_p=False):
                 save_name = file_selection(_('Save As...'), parent=parent,
                         action=gtk.FILE_CHOOSER_ACTION_SAVE)
                 if save_name:
-                    file_p = file(filename, 'rb')
-                    save_p = file(save_name, 'wb+')
+                    file_p = open(filename, 'rb')
+                    save_p = open(save_name, 'wb+')
                     save_p.write(file_p.read())
                     save_p.close()
                     file_p.close()
@@ -334,8 +334,8 @@ def file_open(filename, type, parent, print_p=False):
         save_name = file_selection(_('Save As...'), parent=parent,
                 action=gtk.FILE_CHOOSER_ACTION_SAVE)
         if save_name:
-            file_p = file(filename, 'rb')
-            save_p = file(save_name, 'wb+')
+            file_p = open(filename, 'rb')
+            save_p = open(save_name, 'wb+')
             save_p.write(file_p.read())
             save_p.close()
             file_p.close()
@@ -801,18 +801,26 @@ This record has been modified while you were editing it.
 
 def process_exception(exception, parent, *args):
     global _USERNAME, _DATABASE, _SOCK
+    if str(exception.args[0]) == 'BadFingerprint':
+        warning(_('The server fingerprint has changed since last connection!\n'
+            'The application will stop connecting to this server '
+            'until its fingerprint is fixed.'), parent, _('Security risk!'))
+        from tryton.gui.main import Main
+        Main.sig_quit()
+        sys.exit()
     if str(exception.args[0]) == 'NotLogged':
         if not rpc._SOCK:
             message(_('Connection error!\n' \
                     'Unable to connect to the server!'), parent)
             return False
-        host = rpc._SOCK.host
+        hostname = rpc._SOCK.hostname
         port = rpc._SOCK.port
         while True:
             password = ask(_('Password:'), parent, visibility=False)
             if password is None:
                 raise Exception('NotLogged')
-            res = rpc.login(rpc._USERNAME, password, host, port, rpc._DATABASE)
+            res = rpc.login(rpc._USERNAME, password, hostname, port,
+                    rpc._DATABASE)
             from tryton.gui.main import Main
             Main.get_main().refresh_ssl()
             if res == -1:
@@ -890,9 +898,6 @@ def node_attributes(node):
         return {}
     for i in range(attrs.length):
         result[str(attrs.item(i).localName)] = str(attrs.item(i).nodeValue)
-        if attrs.item(i).localName == "digits" \
-                and isinstance(attrs.item(i).nodeValue, basestring):
-            result[attrs.item(i).localName] = eval(attrs.item(i).nodeValue)
     return result
 
 def hex2rgb(hexstring, digits=2):
@@ -1177,27 +1182,38 @@ _ALLOWED_CODES = set(dis.opmap[x] for x in [
     'BINARY_LSHIFT','BINARY_RSHIFT','BINARY_AND','BINARY_XOR', 'BINARY_OR',
     'STORE_MAP', 'LOAD_NAME', 'CALL_FUNCTION', 'COMPARE_OP', 'LOAD_ATTR',
     'STORE_NAME', 'GET_ITER', 'FOR_ITER', 'LIST_APPEND', 'JUMP_ABSOLUTE',
-    'DELETE_NAME', 'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'BINARY_SUBSCR',
+    'DELETE_NAME', 'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'JUMP_IF_FALSE_OR_POP',
+    'BINARY_SUBSCR',
     ] if x in dis.opmap)
 
+_SAFE_EVAL_CACHE = {}
 
 def safe_eval(source, data=None):
     if '__subclasses__' in source:
         raise ValueError('__subclasses__ not allowed')
-    c = compile(source, '', 'eval')
-    codes = []
-    s = c.co_code
-    i = 0
-    while i < len(s):
-        code = ord(s[i])
-        codes.append(code)
-        if code >= dis.HAVE_ARGUMENT:
-            i += 3
-        else:
-            i += 1
-    for code in codes:
-        if code not in _ALLOWED_CODES:
-            raise ValueError('opcode %s not allowed' % dis.opname[code])
+    if hashlib:
+        key = hashlib.md5(source).digest()
+    else:
+        key = md5.new(source).digest()
+    c = _SAFE_EVAL_CACHE.get(key)
+    if not c:
+        c = compile(source, '', 'eval')
+        codes = []
+        s = c.co_code
+        i = 0
+        while i < len(s):
+            code = ord(s[i])
+            codes.append(code)
+            if code >= dis.HAVE_ARGUMENT:
+                i += 3
+            else:
+                i += 1
+        for code in codes:
+            if code not in _ALLOWED_CODES:
+                raise ValueError('opcode %s not allowed' % dis.opname[code])
+        if len(_SAFE_EVAL_CACHE) > 1024:
+            _SAFE_EVAL_CACHE.clear()
+        _SAFE_EVAL_CACHE[key] = c
     return eval(c, {'__builtins__': {
         'True': True,
         'False': False,
