@@ -21,7 +21,6 @@ import tryton.common as common
 import pango
 from tryton.translate import date_format
 from tryton.common import DT_FORMAT, DHM_FORMAT, HM_FORMAT
-import mx.DateTime
 
 _ = gettext.gettext
 
@@ -53,8 +52,7 @@ class Form(SignalEvent):
 
         self.screen = Screen(self.model, self.window, view_type=view_type,
                 context=self.context, view_ids=view_ids, domain=domain,
-                hastoolbar=CONFIG['form.toolbar'], show_search=True,
-                limit=limit, readonly=bool(auto_refresh), form=self,
+                limit=limit, readonly=bool(auto_refresh),
                 search_value=search_value)
         self.screen.signal_connect(self, 'record-message', self._record_message)
         self.screen.signal_connect(self, 'record-modified', self._record_modified)
@@ -219,16 +217,22 @@ class Form(SignalEvent):
 
     def sig_attach(self, widget=None):
         obj_id = self.id_get()
-        if obj_id >= 0:
+        if obj_id >= 0 and obj_id is not False:
             win = Attachment(self.model, obj_id, self.window)
             win.run()
-            value = self.screen.current_model
-            attachment_count = value.get_attachment_count(reload=True)
-            self.signal('attachment-count', attachment_count)
         else:
             self.message_info(_('No record selected!'))
+        self.update_attachment_count(reload=True)
         return True
 
+    def update_attachment_count(self, reload=False):
+        record = self.screen.current_record
+        if record:
+            attachment_count = record.get_attachment_count(reload=reload)
+        else:
+            attachment_count = 0
+        self.signal('attachment-count', attachment_count)
+
     def sig_switch(self, widget=None):
         if not self.modified_save():
             return
@@ -236,7 +240,7 @@ class Form(SignalEvent):
 
     def sig_logs(self, widget=None):
         obj_id = self.id_get()
-        if obj_id < 0:
+        if obj_id < 0 or obj_id is False:
             self.message_info(_('You have to select one record!'))
             return False
 
@@ -262,25 +266,18 @@ class Form(SignalEvent):
                 if line.get(key, False) \
                         and key in ('create_date', 'write_date'):
                     display_format = date_format() + ' ' + HM_FORMAT
-                    if hasattr(line[key], 'timetuple'):
-                        date = mx.DateTime.DateTime(*(line[key].timetuple()[:6]))
-                    else:
-                        date = mx.DateTime.strptime(str(line[key]), DHM_FORMAT)
+                    date = line[key]
                     if 'timezone' in rpc.CONTEXT:
                         try:
                             import pytz
                             lzone = pytz.timezone(rpc.CONTEXT['timezone'])
                             szone = pytz.timezone(rpc.TIMEZONE)
-                            datetime = DT.datetime(date.year, date.month,
-                                    date.day, date.hour, date.minute,
-                                    int(date.second))
                             sdt = szone.localize(datetime, is_dst=True)
                             ldt = sdt.astimezone(lzone)
-                            date = mx.DateTime.DateTime(*(
-                                ldt.timetuple()[:6]))
+                            date = ldt
                         except:
                             pass
-                    value = date.strftime(display_format)
+                    value = common.datetime_strftime(date, display_format)
                 message_str += val + ' ' + value +'\n'
         message_str += _('Model:') + ' ' + self.model
         message(message_str, self.window)
@@ -298,19 +295,21 @@ class Form(SignalEvent):
                 self.message_info(_('Records removed!'), 'green')
 
     def sig_import(self, widget=None):
-        fields = []
         while(self.screen.view_to_load):
             self.screen.load_view_to_load()
-        win = WinImport(self.model, self.screen.fields, fields,
-                parent=self.window)
+        fields = {}
+        for name, field in self.screen.group.fields.iteritems():
+            fields[name] = field.attrs
+        win = WinImport(self.model, self.window)
         win.run()
 
     def sig_save_as(self, widget=None):
-        fields = []
-        while(self.screen.view_to_load):
+        while self.screen.view_to_load:
             self.screen.load_view_to_load()
-        win = WinExport(self.model, self.ids_get(),
-                self.screen.fields, fields, parent=self.window,
+        fields = {}
+        for name, field in self.screen.group.fields.iteritems():
+            fields[name] = field.attrs
+        win = WinExport(self.model, self.ids_get(), self.window,
                 context=self.context)
         win.run()
 
@@ -375,18 +374,18 @@ class Form(SignalEvent):
         else:
             obj_id = self.id_get()
             self.screen.search_filter()
-            for model in self.screen.models:
-                if model.id == obj_id:
-                    self.screen.current_model = model
-                    self.screen.display()
+            for record in self.screen.group:
+                if record.id == obj_id:
+                    self.screen.current_record = record
+                    self.screen.display(set_cursor=True)
                     break
         self.message_info('')
         return True
 
     def sig_action(self, keyword='form_action'):
         ids = self.ids_get()
-        if self.screen.current_model:
-            obj_id = self.screen.current_model.id
+        if self.screen.current_record:
+            obj_id = self.screen.current_record.id
         else:
             obj_id = False
         if self.screen.current_view.view_type == 'form':
@@ -405,7 +404,7 @@ class Form(SignalEvent):
             if 'active_id' in ctx:
                 del ctx['active_id']
             res = Action.exec_keyword(keyword, self.window, {
-                'model': self.screen.resource,
+                'model': self.screen.model_name,
                 'id': obj_id or False,
                 'ids': ids,
                 }, context=ctx, alwaysask=True)
