@@ -1,6 +1,7 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of this repository contains the full copyright notices and license terms.
+#This file is part of Tryton.  The COPYRIGHT file at the top level of
+#this repository contains the full copyright notices and license terms.
 import time
-import datetime as DT
+import datetime
 import gtk
 import gettext
 import locale
@@ -8,9 +9,8 @@ from interface import WidgetInterface
 import tryton.rpc as rpc
 from tryton.common import DT_FORMAT, DHM_FORMAT, HM_FORMAT, message, \
         TRYTON_ICON
-from tryton.common import date_widget, Tooltips
+from tryton.common import date_widget, Tooltips, datetime_strftime
 from tryton.translate import date_format
-import mx.DateTime
 
 _ = gettext.gettext
 
@@ -18,8 +18,9 @@ _ = gettext.gettext
 class Calendar(WidgetInterface):
     "Calendar"
 
-    def __init__(self, window, parent=None, model=None, attrs=None):
-        super(Calendar, self).__init__(window, parent, model=model, attrs=attrs)
+    def __init__(self, field_name, model_name, window, attrs=None):
+        super(Calendar, self).__init__(field_name, model_name, window,
+                attrs=attrs)
 
         self.format = date_format()
         self.widget = date_widget.ComplexEntry(self.format, spacing=0)
@@ -66,32 +67,32 @@ class Calendar(WidgetInterface):
     def grab_focus(self):
         return self.entry.grab_focus()
 
-    def get_value(self, model):
+    def get_value(self, record):
         value = self.entry.get_text()
         if value == '':
             return False
         try:
-            date = mx.DateTime.strptime(value, self.format)
+            date = datetime.date(*time.strptime(value, self.format)[:3])
         except:
             return False
-        return date.strftime(DT_FORMAT)
+        return datetime_strftime(date, DT_FORMAT)
 
-    def set_value(self, model, model_field):
-        model_field.set_client(model, self.get_value(model))
+    def set_value(self, record, field):
+        field.set_client(record, self.get_value(record))
         return True
 
-    def display(self, model, model_field):
-        if not model_field:
+    def display(self, record, field):
+        if not field:
             self.entry.clear()
             return False
-        super(Calendar, self).display(model, model_field)
-        value = model_field.get_client(model)
+        super(Calendar, self).display(record, field)
+        value = field.get_client(record)
         if not value:
             self.entry.clear()
         else:
             if len(value)>10:
                 value = value[:10]
-            date = mx.DateTime.strptime(value, DT_FORMAT)
+            date = datetime.date(*time.strptime(value, DT_FORMAT)[:3])
             format = self.format
             if date.year < 10:
                 format = format.replace('%Y', '000%Y')
@@ -99,7 +100,7 @@ class Calendar(WidgetInterface):
                 format = format.replace('%Y', '00%Y')
             elif date.year < 1000:
                 format = format.replace('%Y', '0%Y')
-            value = date.strftime(format)
+            value = datetime_strftime(date, format)
             if len(value) > self.entry.get_width_chars():
                 self.entry.set_width_chars(len(value))
             self.entry.set_text(value)
@@ -109,7 +110,7 @@ class Calendar(WidgetInterface):
         return self.entry.get_text()
 
     def cal_open(self, widget):
-        win = gtk.Dialog(_('Date Selection'), self._window,
+        win = gtk.Dialog(_('Date Selection'), self.window,
                 gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                 (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK))
@@ -127,7 +128,7 @@ class Calendar(WidgetInterface):
         win.show_all()
 
         try:
-            val = self.get_value(self.model)
+            val = self.get_value(self.record)
             if val:
                 cal.select_month(int(val[5:7])-1, int(val[0:4]))
                 cal.select_day(int(val[8:10]))
@@ -137,7 +138,7 @@ class Calendar(WidgetInterface):
         response = win.run()
         if response == gtk.RESPONSE_OK:
             year, month, day = cal.get_date()
-            date = mx.DateTime.DateTime(year, month + 1, day)
+            date = datetime.date(year, month + 1, day)
             format = self.format
             if date.year < 10:
                 format = format.replace('%Y', '000%Y')
@@ -145,17 +146,18 @@ class Calendar(WidgetInterface):
                 format = format.replace('%Y', '00%Y')
             elif date.year < 1000:
                 format = format.replace('%Y', '0%Y')
-            self.entry.set_text(date.strftime(format))
+            self.entry.set_text(datetime_strftime(date, format))
         self._focus_out()
-        self._window.present()
+        self.window.present()
         win.destroy()
 
 
 class DateTime(WidgetInterface):
     "DateTime"
 
-    def __init__(self, window, parent, model, attrs=None):
-        super(DateTime, self).__init__(window, parent, model, attrs=attrs)
+    def __init__(self, field_name, model_name, window, attrs=None):
+        super(DateTime, self).__init__(field_name, model_name, window,
+                attrs=attrs)
 
         self.format = date_format() + ' ' + HM_FORMAT
         self.widget = date_widget.ComplexEntry(self.format, spacing=0)
@@ -201,12 +203,12 @@ class DateTime(WidgetInterface):
     def grab_focus(self):
         return self.entry.grab_focus()
 
-    def get_value(self, model, timezone=True):
+    def get_value(self, record, timezone=True):
         value = self.entry.get_text()
         if value == '':
             return False
         try:
-            date = mx.DateTime.strptime(value, self.format)
+            date = datetime.datetime(*time.strptime(value, self.format)[:6])
         except:
             return False
         if 'timezone' in rpc.CONTEXT and timezone:
@@ -214,40 +216,36 @@ class DateTime(WidgetInterface):
                 import pytz
                 lzone = pytz.timezone(rpc.CONTEXT['timezone'])
                 szone = pytz.timezone(rpc.TIMEZONE)
-                datetime = DT.datetime(date.year, date.month, date.day,
-                        date.hour, date.minute, int(date.second))
-                ldt = lzone.localize(datetime, is_dst=True)
+                ldt = lzone.localize(date, is_dst=True)
                 sdt = ldt.astimezone(szone)
-                date = mx.DateTime.DateTime(*(sdt.timetuple()[:6]))
+                date = sdt
             except:
                 pass
-        return date.strftime(DHM_FORMAT)
+        return datetime_strftime(date, DHM_FORMAT)
 
-    def set_value(self, model, model_field):
-        model_field.set_client(model, self.get_value(model))
+    def set_value(self, record, field):
+        field.set_client(record, self.get_value(record))
         return True
 
-    def display(self, model, model_field):
-        super(DateTime, self).display(model, model_field)
-        if not model_field:
+    def display(self, record, field):
+        super(DateTime, self).display(record, field)
+        if not field:
             return self.show(False)
-        self.show(model_field.get_client(model))
+        self.show(field.get_client(record))
 
     def show(self, dt_val, timezone=True):
         if not dt_val:
             self.entry.clear()
         else:
-            date = mx.DateTime.strptime(dt_val, DHM_FORMAT)
+            date = datetime.datetime(*time.strptime(dt_val, DHM_FORMAT)[:6])
             if 'timezone' in rpc.CONTEXT and timezone:
                 try:
                     import pytz
                     lzone = pytz.timezone(rpc.CONTEXT['timezone'])
                     szone = pytz.timezone(rpc.TIMEZONE)
-                    datetime = DT.datetime(date.year, date.month, date.day,
-                            date.hour, date.minute, int(date.second))
-                    sdt = szone.localize(datetime, is_dst=True)
+                    sdt = szone.localize(date, is_dst=True)
                     ldt = sdt.astimezone(lzone)
-                    date = mx.DateTime.DateTime(*(ldt.timetuple()[:6]))
+                    date = ldt
                 except:
                     pass
             format = self.format
@@ -257,7 +255,7 @@ class DateTime(WidgetInterface):
                 format = format.replace('%Y', '00%Y')
             elif date.year < 1000:
                 format = format.replace('%Y', '0%Y')
-            value = date.strftime(format)
+            value = datetime_strftime(date, format)
             if len(value) > self.entry.get_width_chars():
                 self.entry.set_width_chars(len(value))
             self.entry.set_text(value)
@@ -267,7 +265,7 @@ class DateTime(WidgetInterface):
         return self.entry.get_text()
 
     def cal_open(self, widget):
-        win = gtk.Dialog(_('Date Time Selection'), self._window,
+        win = gtk.Dialog(_('Date Time Selection'), self.window,
                 gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                 (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK))
@@ -296,7 +294,7 @@ class DateTime(WidgetInterface):
         win.show_all()
 
         try:
-            val = self.get_value(self.model, timezone=False)
+            val = self.get_value(self.record, timezone=False)
             if val:
                 widget_hour.set_value(int(val[11:13]))
                 widget_minute.set_value(int(val[-5:-3]))
@@ -314,9 +312,9 @@ class DateTime(WidgetInterface):
             year = int(cal.get_date()[0])
             month = int(cal.get_date()[1]) + 1
             day = int(cal.get_date()[2])
-            date = mx.DateTime.DateTime(year, month, day, hour, minute)
-            value = date.strftime(DHM_FORMAT)
+            date = datetime.datetime(year, month, day, hour, minute)
+            value = datetime_strftime(date, DHM_FORMAT)
             self.show(value, timezone=False)
         self._focus_out()
-        self._window.present()
+        self.window.present()
         win.destroy()
