@@ -40,16 +40,16 @@ class Button(object):
     def button_clicked(self, widget):
         if not self.form:
             return
-        model = self.form.screen.current_model
+        record = self.form.screen.current_record
         obj_id = self.form.screen.save_current()
         if obj_id:
             if not self.attrs.get('confirm', False) or \
                     common.sur(self.attrs['confirm'], self.form.window):
                 button_type = self.attrs.get('type', 'workflow')
                 ctx = rpc.CONTEXT.copy()
-                ctx.update(model.context_get())
+                ctx.update(record.context_get())
                 if button_type == 'workflow':
-                    args = ('model', self.form.screen.name,
+                    args = ('model', self.form.screen.model_name,
                             'workflow_trigger_validate', obj_id,
                             self.attrs['name'], ctx)
                     try:
@@ -58,7 +58,7 @@ class Button(object):
                         common.process_exception(exception, self.form.window,
                                 *args)
                 elif button_type == 'object':
-                    args = ('model', self.form.screen.name, self.attrs['name'],
+                    args = ('model', self.form.screen.model_name, self.attrs['name'],
                             [obj_id], ctx)
                     try:
                         rpc.execute(*args)
@@ -76,7 +76,7 @@ class Button(object):
                                 *args)
                     if action_id:
                         Action.execute(action_id, {
-                            'model': self.form.screen.name,
+                            'model': self.form.screen.model_name,
                             'id': obj_id,
                             'ids': [obj_id],
                             }, self.form.window, context=ctx)
@@ -84,110 +84,48 @@ class Button(object):
                     raise Exception('Unallowed button type')
                 self.form.screen.reload(writen=True)
         else:
-            if self.form.screen.form:
-                self.form.screen.form.message_info(
-                        _('Invalid Form, correct red fields!'))
             self.form.screen.display()
 
-    def state_set(self, model):
-        state_changes = self.attrs.get('states', {})
-        if isinstance(state_changes, basestring):
-            try:
-                state_changes = common.safe_eval(state_changes)
-            except:
-                self.widget.show()
-                self.widget.set_sensitive(True)
-                return
-        if 'invisible' in state_changes:
-            try:
-                if model:
-                    if model.expr_eval(state_changes['invisible'],
-                        check_load=False):
-                        self.widget.hide()
-                    else:
-                        self.widget.show()
-            except:
-                log = logging.getLogger('record')
-                log.error("Unable to eval '%s' for button %s (record: %s@%s)"% \
-                              (state_changes['invisible'],
-                               self.attrs.get('string', _('Unknown')),
-                               model and model.id or _('Unknown'),
-                               model and model.resource or _('Unknown'),
-                               ))
-                self.widget.show()
+    def state_set(self, record):
+        if record:
+            state_changes = record.expr_eval(self.attrs.get('states', {}),
+                    check_load=False)
+        else:
+            state_changes = {}
+        if state_changes.get('invisible', False):
+            self.widget.hide()
         else:
             self.widget.show()
-        if 'readonly' in state_changes:
-            try:
-                if model and model.expr_eval(state_changes['readonly'],
-                        check_load=False):
-                    self.widget.set_sensitive(False)
-                else:
-                    self.widget.set_sensitive(True)
-            except:
-                log = logging.getLogger('record')
-                log.error("Unable to eval '%s' for button %s (record: %s@%s)"% \
-                              (state_changes['readonly'],
-                               self.attrs.get('string', _('Unknown')),
-                               model and model.id or _('Unknown'),
-                               model and model.resource or _('Unknown'),
-                               ))
-
-                self.widget.set_sensitive(True)
-        else:
-            self.widget.set_sensitive(True)
+        self.widget.set_sensitive(not state_changes.get('readonly', False))
         if 'icon' in state_changes:
-            try:
-                stock = model and model.expr_eval(
-                    state_changes['icon'], check_load=False)
-                if stock:
-                    try:
-                        icon = gtk.Image()
-                        icon.set_from_stock(stock, gtk.ICON_SIZE_SMALL_TOOLBAR)
-                        self.widget.set_image(icon)
-                    except:
-                        log = logging.getLogger('common')
-                        log.warning(_('Wrong icon for the button!'))
-                else:
-                    self.widget.set_image(gtk.Image())
-            except:
-                log = logging.getLogger('record')
-                log.error("Unable to eval '%s' for button %s (record: %s@%s)"% \
-                              (state_changes['icon'],
-                               self.attrs.get('string', _('Unknown')),
-                               model and model.id or _('Unknown'),
-                               model and model.resource or _('Unknown'),
-))
+            stock = state_changes['icon']
+            if stock:
+                try:
+                    icon = gtk.Image()
+                    icon.set_from_stock(stock, gtk.ICON_SIZE_SMALL_TOOLBAR)
+                    self.widget.set_image(icon)
+                except:
+                    log = logging.getLogger('common')
+                    log.warning(_('Wrong icon for the button!'))
+            else:
                 self.widget.set_image(gtk.Image())
 
+
 class Label(gtk.Label):
 
     def __init__(self, str=None, attrs=None):
         super(Label, self).__init__(str=str)
         self.attrs = attrs or {}
 
-    def state_set(self, model):
-        state_changes = self.attrs.get('states', {})
-        try:
-            if isinstance(state_changes, basestring):
-                state_changes = common.safe_eval(state_changes)
-            if 'invisible' in state_changes:
-                if model:
-                    if model.expr_eval(state_changes['invisible'],
-                        check_load=False):
-                        self.hide()
-                    else:
-                        self.show()
-            else:
-                self.show()
-        except:
-            log = logging.getLogger('record')
-            log.error("Unable to eval '%s' for label %s (record: %s@%s)"% \
-                          (state_changes['invisible'],
-                           self.attrs.get('string', _('Unknown')),
-                           model and model.id or _('Unknown'),
-                           model and model.resource or _('Unknown'),
-                           ))
+    def state_set(self, record):
+        if record:
+            state_changes = record.expr_eval(self.attrs.get('states', {}),
+                    check_load=False)
+        else:
+            state_changes = {}
+        if state_changes.get('invisible', self.attrs.get('invisible')):
+            self.hide()
+        else:
             self.show()
 
 
@@ -197,89 +135,57 @@ class VBox(gtk.VBox):
         super(VBox, self).__init__(homogeneous, spacing)
         self.attrs = attrs or {}
 
-    def state_set(self, model):
-        state_changes = self.attrs.get('states', {})
-        try:
-            if isinstance(state_changes, basestring):
-                state_changes = common.safe_eval(state_changes)
-            if 'invisible' in state_changes:
-                if model:
-                    if model.expr_eval(state_changes['invisible'],
-                        check_load=False):
-                        self.hide()
-                    else:
-                        self.show()
-            else:
-                self.show()
-        except:
-            log = logging.getLogger('record')
-            log.error("Unable to eval '%s' for separator %s (record: %s@%s)"% \
-                          (state_changes['invisible'],
-                           self.attrs.get('string', _('Unknown')),
-                           model and model.id or _('Unknown'),
-                           model and model.resource or _('Unknown'),
-                           ))
+    def state_set(self, record):
+        if record:
+            state_changes = record.expr_eval(self.attrs.get('states', {}),
+                    check_load=False)
+        else:
+            state_changes = {}
+        if state_changes.get('invisible', self.attrs.get('invisible')):
+            self.hide()
+        else:
             self.show()
 
+
 class Image(gtk.Image):
 
     def __init__(self, attrs=None):
         super(Image, self).__init__()
         self.attrs = attrs or {}
 
-    def state_set(self, model):
-        state_changes = self.attrs.get('states', {})
-        try:
-            if isinstance(state_changes, basestring):
-                state_changes = common.safe_eval(state_changes)
-            if 'invisible' in state_changes:
-                if model:
-                    if model.expr_eval(state_changes['invisible'],
-                        check_load=False):
-                        self.hide()
-                    else:
-                        self.show()
-            else:
-                self.show()
-        except:
-            log = logging.getLogger('record')
-            log.error("Unable to eval '%s' for image %s (record: %s@%s)"% \
-                          (state_changes['invisible'],
-                           self.attrs.get('string', _('Unknown')),
-                           model and model.id or _('Unknown'),
-                           model and model.resource or _('Unknown'),
-                           ))
+    def state_set(self, record):
+        if record:
+            state_changes = record.expr_eval(self.attrs.get('states', {}),
+                    check_load=False)
+        else:
+            state_changes = {}
+        if state_changes.get('invisible', self.attrs.get('invisible')):
+            self.hide()
+        else:
             self.show()
+        state_changes = self.attrs.get('states', {})
 
 
 class Frame(gtk.Frame):
 
     def __init__(self, label=None, attrs=None):
+        if not label: # label must be None to have no label widget
+            label = None
         super(Frame, self).__init__(label=label)
         self.attrs = attrs or {}
+        if not label:
+            self.set_shadow_type(gtk.SHADOW_NONE)
+        self.set_border_width(0)
 
-    def state_set(self, model):
-        state_changes = self.attrs.get('states', {})
-        try:
-            if isinstance(state_changes, basestring):
-                state_changes = common.safe_eval(state_changes)
-            if 'invisible' in state_changes:
-                if model:
-                    if model.expr_eval(state_changes['invisible'],
-                        check_load=False):
-                        self.hide()
-                    else:
-                        self.show()
-            else:
-                self.show()
-        except:
-            log = logging.getLogger('record')
-            log.error("Unable to eval '%s' for group %s (record: %s@%s)"% \
-                          (state_changes['invisible'],
-                           self.attrs.get('string', _('Unknown')),
-                           model and model.id or _('Unknown'),
-                           model and model.resource or _('Unknown'),
-                           ))
+    def state_set(self, record):
+        if record:
+            state_changes = record.expr_eval(self.attrs.get('states', {}),
+                    check_load=False)
+        else:
+            state_changes = {}
+        if state_changes.get('invisible', self.attrs.get('invisible')):
+            self.hide()
+        else:
             self.show()
 
 
@@ -290,28 +196,15 @@ class ScrolledWindow(gtk.ScrolledWindow):
                 vadjustment=vadjustment)
         self.attrs = attrs or {}
 
-    def state_set(self, model):
-        state_changes = self.attrs.get('states', {})
-        try:
-            if isinstance(state_changes, basestring):
-                state_changes = common.safe_eval(state_changes)
-            if 'invisible' in state_changes:
-                if model:
-                    if model.expr_eval(state_changes['invisible'],
-                        check_load=False):
-                        self.hide()
-                    else:
-                        self.show()
-            else:
-                self.show()
-        except:
-            log = logging.getLogger('record')
-            log.error("Unable to eval '%s' for page %s (record: %s@%s)"% \
-                          (state_changes['invisible'],
-                           self.attrs.get('string', _('Unknown')),
-                           model and model.id or _('Unknown'),
-                           model and model.resource or _('Unknown'),
-                           ))
+    def state_set(self, record):
+        if record:
+            state_changes = record.expr_eval(self.attrs.get('states', {}),
+                    check_load=False)
+        else:
+            state_changes = {}
+        if state_changes.get('invisible', self.attrs.get('invisible')):
+            self.hide()
+        else:
             self.show()
 
 
@@ -402,7 +295,7 @@ class ParserForm(ParserInterface):
                 screen=screen)
         self.widget_id = 0
 
-    def parse(self, model, root_node, fields, notebook=None, paned=None,
+    def parse(self, model_name, root_node, fields, notebook=None, paned=None,
             tooltips=None):
         dict_widget = {}
         button_list = []
@@ -427,7 +320,7 @@ class ParserForm(ParserInterface):
             attrs = common.node_attributes(node)
             if not cursor_widget:
                 if attrs.get('name') and fields.get(attrs['name']) \
-                        and not fields[attrs['name']].get('exclude_field',
+                        and not fields[attrs['name']].attrs.get('exclude_field',
                                 False):
                     cursor_widget = attrs.get('name')
             if node.localName == 'image':
@@ -440,13 +333,14 @@ class ParserForm(ParserInterface):
                         fill=int(attrs.get('fill', 0)))
             elif node.localName == 'separator':
                 text = attrs.get('string', '')
-                if 'string' in attrs or 'name' in attrs:
+                if 'name' in attrs:
+                    for attr_name in ('states', 'invisible'):
+                        if attr_name not in attrs and attrs['name'] in fields:
+                            if attr_name in fields[attrs['name']].attrs:
+                                attrs[attr_name] = fields[attrs['name']
+                                        ].attrs[attr_name]
                     if not text:
-                        if 'name' in attrs and attrs['name'] in fields:
-                            if 'states' in fields[attrs['name']]:
-                                attrs['states'] = \
-                                        fields[attrs['name']]['states']
-                            text = fields[attrs['name']]['string']
+                        text = fields[attrs['name']].attrs['string']
                 vbox = VBox(attrs=attrs)
                 button_list.append(vbox)
                 if text:
@@ -461,22 +355,28 @@ class ParserForm(ParserInterface):
                         fill=int(attrs.get('fill', 0)))
             elif node.localName == 'label':
                 text = attrs.get('string', '')
-                if not text:
-                    if 'name' in attrs and attrs['name'] in fields:
-                        if 'states' in fields[attrs['name']]:
-                            attrs['states'] = fields[attrs['name']]['states']
-                        if gtk.widget_get_default_direction() == gtk.TEXT_DIR_RTL:
-                            text = _(':') + fields[attrs['name']]['string']
+                if 'name' in attrs and attrs['name'] in fields:
+                    for attr_name in ('states', 'invisible'):
+                        if attr_name not in attrs \
+                                and attr_name in fields[attrs['name']].attrs:
+                            attrs[attr_name] = fields[attrs['name']
+                                    ].attrs[attr_name]
+                    if not text:
+                        if gtk.widget_get_default_direction() == \
+                                gtk.TEXT_DIR_RTL:
+                            text = _(':') + \
+                                    fields[attrs['name']].attrs['string']
                         else:
-                            text = fields[attrs['name']]['string'] + _(':')
-                        if 'align' not in attrs:
-                            attrs['align'] = 1.0
-                    else:
-                        for node in node.childNodes:
-                            if node.nodeType == node.TEXT_NODE:
-                                text += node.data
-                            else:
-                                text += node.toxml()
+                            text = fields[attrs['name']].attrs['string'] + \
+                                    _(':')
+                    if 'align' not in attrs:
+                        attrs['align'] = 1.0
+                elif not text:
+                    for node in node.childNodes:
+                        if node.nodeType == node.TEXT_NODE:
+                            text += node.data
+                        else:
+                            text += node.toxml()
                 if not text:
                     container.empty_add(int(attrs.get('colspan', 1)))
                     continue
@@ -539,11 +439,11 @@ class ParserForm(ParserInterface):
                 container.wid_add(notebook, colspan=attrs.get('colspan', 4),
                         expand=True, fill=True)
                 widget, widgets, buttons, spam, notebook_list2, cursor_widget2 = \
-                        self.parse(model, node, fields, notebook,
+                        self.parse(model_name, node, fields, notebook,
                                 tooltips=tooltips)
                 max_width, max_height = -1, -1
                 window_width, window_height = Main.get_main().window.get_size()
-                for i in range(notebook.get_n_pages()):
+                for i in xrange(notebook.get_n_pages()):
                     width, height = notebook.get_nth_page(i)\
                             .get_child().get_child().size_request()
                     if width > max_width and width < window_width - 50:
@@ -568,11 +468,12 @@ class ParserForm(ParserInterface):
                     angle = 0
                 text = attrs.get('string', '')
                 if 'name' in attrs and attrs['name'] in fields:
-                    if 'states' in fields[attrs['name']]:
-                        attrs['states'] = \
-                                fields[attrs['name']]['states']
+                    for attr_name in ('states', 'invisible'):
+                        if attr_name in fields[attrs['name']].attrs:
+                            attrs[attr_name] = \
+                                    fields[attrs['name']].attrs[attr_name]
                     if not text:
-                        text = fields[attrs['name']]['string']
+                        text = fields[attrs['name']].attrs['string']
                 if not text:
                     text = _('No String Attr.')
                 if '_' not in text:
@@ -581,13 +482,13 @@ class ParserForm(ParserInterface):
                 label.set_angle(angle)
                 label.set_use_underline(True)
                 widget, widgets, buttons, spam, notebook_list2, cursor_widget2 = \
-                        self.parse(model, node, fields, notebook,
+                        self.parse(model_name, node, fields, notebook,
                                 tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
                 button_list += buttons
-                for widget_name, widgets in widgets.items():
+                for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
 
@@ -606,24 +507,24 @@ class ParserForm(ParserInterface):
 
             elif node.localName == 'field':
                 name = str(attrs['name'])
-                del attrs['name']
                 if name not in fields:
                     container.empty_add(int(attrs.get('colspan', 1)))
                     log = logging.getLogger('view')
                     log.error('Unknown field "%s"' % str(name))
                     continue
-                ftype = attrs.get('widget', fields[name]['type'])
-                fields[name].update(attrs)
-                fields[name]['model'] = model
+                ftype = attrs.get('widget', fields[name].attrs['type'])
                 if not ftype in WIDGETS_TYPE:
                     container.empty_add(int(attrs.get('colspan', 1)))
                     continue
+                for attr_name in ('relation', 'domain', 'selection',
+                        'relation_field', 'string', 'views', 'invisible',
+                        'add_remove', 'sort'):
+                    if attr_name in fields[name].attrs and \
+                            not attr_name in attrs:
+                        attrs[attr_name] = fields[name].attrs[attr_name]
 
-                fields[name]['name'] = name
-                if 'saves' in attrs:
-                    fields[name]['saves'] = attrs['saves']
-                widget_act = WIDGETS_TYPE[ftype][0](self.window, self.parent,
-                        model, fields[name])
+                widget_act = WIDGETS_TYPE[ftype][0](name, model_name,
+                        self.window, attrs)
                 self.widget_id += 1
                 widget_act.position = self.widget_id
                 dict_widget.setdefault(name, [])
@@ -641,42 +542,42 @@ class ParserForm(ParserInterface):
                 xfill = True
                 if 'xfill' in attrs:
                     xfill = bool(common.safe_eval(attrs['xfill']))
-                hlp = fields[name].get('help', attrs.get('help', False))
+                hlp = fields[name].attrs.get('help', attrs.get('help', False))
                 if attrs.get('height', False) or attrs.get('width', False):
                     widget_act.widget.set_size_request(
                             int(attrs.get('width', -1)),
                             int(attrs.get('height', -1)))
                 translate = False
                 if ftype in ('char', 'text'):
-                    translate = fields[name].get('translate', False)
-                container.wid_add(widget_act.widget, fields[name]['string'],
-                        expand, translate=translate, colspan=size, fname=name,
+                    translate = fields[name].attrs.get('translate', False)
+                container.wid_add(widget_act.widget,
+                        fields[name].attrs['string'], expand,
+                        translate=translate, colspan=size, fname=name,
                         help_tip=hlp, fill=fill, xexpand=xexpand, xfill=xfill)
 
             elif node.localName == 'group':
                 widget, widgets, buttons, spam, notebook_list2, cursor_widget2 = \
-                        self.parse(model, node, fields, tooltips=tooltips)
+                        self.parse(model_name, node, fields, tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
-                for widget_name, widgets in widgets.items():
+                for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
                 button_list += buttons
                 text = ''
                 if 'name' in attrs and attrs['name'] in fields:
-                    if 'states' in fields[attrs['name']]:
-                        attrs['states'] = fields[attrs['name']]['states']
-                    text = fields[attrs['name']]['string']
+                    for attr_name in ('states', 'invisible'):
+                        if attr_name in fields[attrs['name']].attrs:
+                            attrs[attr_name] = fields[attrs['name']
+                                    ].attrs[attr_name]
+                    text = fields[attrs['name']].attrs['string']
                 if attrs.get('string'):
                     text = attrs['string']
 
-                if text:
-                    frame = Frame(text, attrs)
-                    frame.add(widget)
-                    button_list.append(frame)
-                else:
-                    frame = widget
+                frame = Frame(text, attrs)
+                frame.add(widget)
+                button_list.append(frame)
                 container.wid_add(frame, colspan=int(attrs.get('colspan', 1)),
                         expand=int(attrs.get('expand', 0)),
                         rowspan=int(attrs.get('rowspan', 1)), ypadding=0,
@@ -686,13 +587,13 @@ class ParserForm(ParserInterface):
                 container.wid_add(hpaned, colspan=int(attrs.get('colspan', 4)),
                         expand=True, fill=True)
                 widget, widgets, buttons, spam, notebook_list2, cursor_widget2 = \
-                        self.parse(model, node, fields, paned=hpaned,
+                        self.parse(model_name, node, fields, paned=hpaned,
                                 tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
                 button_list += buttons
-                for widget_name, widgets in widgets.items():
+                for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
                 if 'position' in attrs:
@@ -702,56 +603,47 @@ class ParserForm(ParserInterface):
                 container.wid_add(vpaned, colspan=int(attrs.get('colspan', 4)),
                         expand=True, fill=True)
                 widget, widgets, buttons, spam, notebook_list, cursor_widget2 = \
-                        self.parse(model, node, fields, paned=vpaned,
+                        self.parse(model_name, node, fields, paned=vpaned,
                                 tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
                 button_list += buttons
-                for widget_name, widgets in widgets.items():
+                for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
                 if 'position' in attrs:
                     vpaned.set_position(int(attrs['position']))
-            elif node.localName == 'child1':
+            elif node.localName == 'child':
                 widget, widgets, buttons, spam, notebook_list2, cursor_widget2 = \
-                        self.parse(model, node, fields, paned=paned,
+                        self.parse(model_name, node, fields, paned=paned,
                                 tooltips=tooltips)
                 if not cursor_widget:
                     cursor_widget = cursor_widget2
                 notebook_list.extend(notebook_list2)
                 button_list += buttons
-                for widget_name, widgets in widgets.items():
+                for widget_name, widgets in widgets.iteritems():
                     dict_widget.setdefault(widget_name, [])
                     dict_widget[widget_name].extend(widgets)
-                paned.pack1(widget, resize=True, shrink=True)
-            elif node.localName == 'child2':
-                widget, widgets, buttons, spam, notebook_list, cursor_widget2 = \
-                        self.parse(model, node, fields, paned=paned,
-                                tooltips=tooltips)
-                if not cursor_widget:
-                    cursor_widget = cursor_widget2
-                notebook_list.extend(notebook_list2)
-                button_list += buttons
-                for widget_name, widgets in widgets.items():
-                    dict_widget.setdefault(widget_name, [])
-                    dict_widget[widget_name].extend(widgets)
-                paned.pack2(widget, resize=True, shrink=True)
+                if not paned.get_child1():
+                    paned.pack1(widget, resize=True, shrink=True)
+                elif not paned.get_child2():
+                    paned.pack2(widget, resize=True, shrink=True)
         for (button, src, name, widget) in container.trans_box:
-            button.connect('clicked', self.translate, model, name,
+            button.connect('clicked', self.translate, model_name, name,
                     src, widget)
         return container.pop(), dict_widget, button_list, on_write, \
                 notebook_list, cursor_widget
 
-    def translate(self, widget, model, name, src, widget_entry):
-        obj_id = self.screen.current_model.id
+    def translate(self, widget, model_name, name, src, widget_entry):
+        obj_id = self.screen.current_record.id
         if obj_id < 0:
             common.message(
                     _('You need to save the record before adding translations!'),
                     parent=self.window)
             return False
 
-        obj_id = self.screen.current_model.save(force_reload=False)
+        obj_id = self.screen.current_record.save(force_reload=False)
         try:
             lang_ids = rpc.execute('model', 'ir.lang',
                     'search', [('translatable', '=', '1')])
@@ -850,7 +742,7 @@ class ParserForm(ParserInterface):
             context = copy.copy(rpc.CONTEXT)
             context['language'] = lang['code']
             try:
-                val = rpc.execute('model', model,
+                val = rpc.execute('model', model_name,
                         'read', [obj_id], [name], context)
             except Exception, exception:
                 common.process_exception(exception, self.window)
@@ -901,7 +793,7 @@ class ParserForm(ParserInterface):
                     value_set(widget_entry, new_val['value'])
                 context = copy.copy(rpc.CONTEXT)
                 context['language'] = new_val['code']
-                args = ('model', model, 'write', [obj_id],
+                args = ('model', model_name, 'write', [obj_id],
                         {str(name):  new_val['value']}, context)
                 try:
                     rpc.execute(*args)
@@ -911,7 +803,7 @@ class ParserForm(ParserInterface):
             self.window.present()
             win.destroy()
             return
-        self.screen.current_model.reload()
+        self.screen.current_record.reload()
         self.window.present()
         win.destroy()
         return True
