@@ -55,7 +55,7 @@ class Dialog(object):
                 val[i] = fields[i]['value']
 
         self.screen = Screen(obj_name, self.dia, view_type=[], context=context)
-        self.screen.add_view_custom(arch, fields, display=True)
+        self.screen.add_view(arch, fields, display=True)
 
         title = gtk.Label()
         title.set_use_markup(True)
@@ -110,7 +110,7 @@ class Dialog(object):
         self.dia.set_title(self.screen.current_view.title)
         self.dia.show()
         self.screen.new(default=False)
-        self.screen.current_model.set(val)
+        self.screen.current_record.set_default(val)
         self.screen.current_view.set_cursor()
 
     def run(self, datas=None):
@@ -119,7 +119,7 @@ class Dialog(object):
         while True:
             res = self.dia.run()
             self.screen.current_view.set_value()
-            if self.screen.current_model.validate() \
+            if self.screen.current_record.validate() \
                     or (res<0) or (self.states[res]=='end'):
                 break
             self.screen.display()
@@ -138,7 +138,7 @@ class Dialog(object):
             datas.update(self.screen.get())
             self.dia.hide()
             self.parent.present()
-            return (self.states[res], datas)
+            return self.states[res], datas
         else:
             self.dia.hide()
             self.parent.present()
@@ -175,6 +175,7 @@ class Wizard(object):
             if not wiz_id:
                 return
         dia = None
+        res = {}
         while state != 'end':
             ctx = context.copy()
             ctx.update(rpc.CONTEXT)
@@ -184,16 +185,20 @@ class Wizard(object):
                 action, 'execute', wiz_id, datas, state, ctx), parent)
             try:
                 res = rpcprogress.run()
+                exception = None
             except Exception, exception:
                 common.process_exception(exception, parent)
-                break
+                # Continue by running previous result except if access to
+                # wizard is denied
+                if exception.args[0] == 'AccessDenied':
+                    break
             if not res:
                 if dia:
                     res = {'type': 'form'}
                 else:
                     break
             else:
-                if dia:
+                if dia and not exception:
                     dia.destroy()
                     dia = None
 
@@ -206,11 +211,12 @@ class Wizard(object):
                     dia = Dialog(res['arch'], res['fields'], res['state'],
                             res['object'], parent, action=action,
                             size=res['size'], context=ctx)
-                    dia.screen.current_model.set(datas['form'])
+                    dia.screen.current_record.set(datas['form'])
                 res2 = dia.run(datas['form'])
                 if not res2:
                     break
-                (state, new_data) = res2
+                state, new_data = res2
+
                 for data in new_data:
                     if new_data[data] is None:
                         del new_data[data]
